const uuidv4 = require('uuid/v4');
const system = require("../../../system");
const ServiceBase = require("../../sve.base");
const settings = require("../../../../config/settings");
var moment = require('moment')
class NeedinfoService extends ServiceBase {
  constructor () {
    super("dbneed", ServiceBase.getDaoName(NeedinfoService));
    this.execlient = system.getObject("util.execClient");
    this.needsolutionSve = system.getObject("service.dbneed.needsolutionSve");
    this.needsolutionDao = system.getObject("db.dbneed.needsolutionDao");
    this.needinfoDao = system.getObject("db.dbneed.needinfoDao");
  }
  async getItemByNeedNo (pobj) {
    var item = await this.dao.getItemByNeedNo(pobj.actionBody.needNo);
    console.log(system.getResultSuccess(item));
    if (!item) {
      return system.getResult(null, "需求数据为空，30210");
    }
    return system.getResultSuccess(item);
  }

  async opSubmitNeed (pobj, actionBody, req) {
    var needNo = await this.getBusUid("n");
    if (!actionBody.mobile) {
      return system.getResultFail(-5002, "mobile不能为空");
    }
    // if (!actionBody.type) {
    //   return system.getResultFail(-5003, "type不能为空");
    // }
    var nobj = {
      uapp_id: pobj.appInfo.uapp_id,
      channelNeedNo: needNo, // 2020 0807 lin 修改内容 渠道需求号改为企服通需求号 不再单独获取新的uuid
      needNo: needNo,
      channelUserId: pobj.userInfo.channel_userid,
      publishName: actionBody.userName,
      publishContent: actionBody.description,
      publishMobile: actionBody.mobile,
      notes: actionBody.description,// 2020 0814 lin新增 插入备注，格式：公司注册，商标注册。目前用来作为抖音的打包下需求功能。用来展示在个人中心/运营后台需求列表
      city: actionBody.area,
      typeCode: actionBody.type_code,
      typeName: actionBody.type_name,
      channelTypeCode: actionBody.channel_type_code,
      channelTypeName: actionBody.channel_type_name,
      status: "wts"
    }
    var r = await this.dao.create(nobj);
    return system.getResultSuccess(r);
  }

  async opNeedList (pobj, actionBody, req) {
    // 时间段，分页，排序，需求状态 条件展示未做
    let statusDic = ['wts', 'yts', 'ygj', 'ygb', 'ycd']
    console.log(pobj);
    console.log(pobj.actionBody);
    var sobj = {
      search: {
        channelUserId: pobj.userInfo.channel_userid
      },
      pageInfo: {
        pageNo: Number(actionBody.pageNumber) || 1,
        pageSize: Number(actionBody.pageSize) || 10
      }
    }
    // if (actionBody.consultType) {
    //   sobj.search.consult_type = actionBody.consultType;
    // }
    // 2020 0924 lin 修改之前查询条件consultType 为 channelTypeCode
    // 注意这里 对应数据库的是typeCode 但传来的参数是consultType
    if (actionBody.consultType) {
      sobj.search.typeCode = actionBody.consultType;
    }
    // liang  追加筛选条件
    if (actionBody.status && statusDic.indexOf(actionBody.status) != -1) {
      sobj.search.status = actionBody.status;
    }
    if (actionBody.needNo) {
      sobj.search.needNo = actionBody.needNo
    }
    if (actionBody.publishName) {
      sobj.search.publishName = actionBody.publishName
    }
    if (actionBody.publishMobile) {
      sobj.search.publishMobile = actionBody.publishMobile
    }
    console.log(actionBody)
    if (actionBody.stdate && actionBody.endate) {
      sobj.search.updated_at = [actionBody.stdate, actionBody.endate]
    }
    var r = await this.dao.findAndCountAll(sobj);
    return system.getResultSuccess(r);
  }

  async opNeedListBak (pobj, actionBody, req) {
    let statusDic = ['wts', 'yts', 'ygj', 'ygb', 'ycd']
    var sobj = {
      search: {
      },
      pageInfo: {
        pageNo: Number(actionBody.pageNo) || 1,
        pageSize: Number(actionBody.pageSize) || 10
      }
    }
    if (pobj.appInfo.uapp_id) {
      sobj.search.uapp_id = pobj.appInfo.uapp_id
    }
    if (actionBody.typeCode) {
      sobj.search.typeCode = actionBody.typeCode;
    }
    if (actionBody.status && statusDic.indexOf(actionBody.status) != -1) {
      sobj.search.status = actionBody.status;
    }
    if (actionBody.needNo) {
      sobj.search.needNo = actionBody.needNo
    }
    if (actionBody.publishName) {
      sobj.search.publishName = actionBody.publishName
    }
    if (actionBody.publishMobile) {
      sobj.search.publishMobile = actionBody.publishMobile
    }
    if (actionBody.stdate && actionBody.endate) {
      sobj.search.updated_at = [actionBody.stdate, actionBody.endate]
    }
    var r = await this.dao.findAndCountAll(sobj);
    return system.getResultSuccess(r);
  }

  async opNeedClose (pobj, actionBody, req) {
    var needinfo = await this.findOne({ channelNeedNo: actionBody.needNo });
    if (!needinfo) {
      return system.getResultFail(-5004, "需求不存在");
    }
    if (needinfo.status == "ygb" || needinfo.status == "ycd") {
      return system.getResultSuccess();
    } else {
      needinfo.status = "ygb";
      var self = this;
      await self.update(needinfo.dataValues);
      return system.getResultSuccess();
    }
  }

  /**
   * 各渠道需求统计
   * @param pobj
   * @returns {Promise<void>}
   */
  async getStatisticsByUappId(pobj){
    let ac = pobj.actionBody;
    let sql = `SELECT uapp_id,count(*) count,DATE_FORMAT(created_at,'%Y-%m-%d') time FROM n_need_info WHERE deleted_at is null `;
    let whereParam = {};
    if(ac.start&&ac.end){
      sql += ` AND created_at >= :start AND created_at <= :end`;
      whereParam.start = ac.start;
      whereParam.end = ac.end;
    }
    if(ac.type_code){
      sql += ` AND typeCode = :type_code`;
      whereParam.type_code = ac.type_code;
    }
    if(ac.status){
      sql += ` AND status = :status`;
      whereParam.status = ac.status;
    }
    sql += ` GROUP BY uapp_id,DATE_FORMAT(created_at,'%Y-%m-%d') ORDER BY created_at ASC`
    let result = await this.customQuery(sql,whereParam);
    return system.getResultSuccess(result);
  }

  /**
   * 需求统计漏斗图
   * @param pobj
   * @returns {Promise<void>}
   */
  async getNeedFunnelStatistics(pobj){
      let ac = pobj.actionBody;
      let sql = `select count(*) count from n_need_info where deleted_at is null `;
      let sql2 = `select count(DISTINCT(a.needNo)) count from n_need_solution a left join n_need_info b on a.needNo = b.needNo where a.deleted_at is null`;
      let whereParams = {};
      if (ac.start&&ac.end){
        sql += ` AND created_at >= :start and created_at <= :end`;
        sql2 += ` AND b.created_at >= :start and b.created_at <= :end`;
        whereParams.start = ac.start;
        whereParams.end = ac.end;
      }
      if(ac.uapp_id){
        sql += ` and uapp_id = :uapp_id`;
        sql2 +=  ` and b.uapp_id = :uapp_id`;
        whereParams.uapp_id = ac.uapp_id;
      }
      if(ac.type_code){
        sql += ` and typeCode = :type_code`;
        sql2 += ` and b.typeCode = :type_code`;
        whereParams.type_code = ac.type_code;
      }
      let total = await this.customQuery(sql,whereParams);
      let middle = await this.customQuery(sql2,whereParams);
      let sql3 = sql2 + ` and b.status = 'ycd'`
      let finish = await this.customQuery(sql3,whereParams);
      let result = {
        total:total[0].count,
        middle:middle[0].count,
        finish:finish[0].count
      }
      return system.getResult(result);
  }

  /**
   * 需求统计（产品维度）
   * @param pobj
   * @returns {Promise<void>}
   */
  async getStatisticsByProduct(pobj){
    let ac = pobj.actionBody;
    let sql = `select typeCode,uapp_id,count(*) count from n_need_info where typeCode is not null `;
    let whereParams = {};
    if(ac.start&&ac.end){
      sql += ` and created_at >= :start and created_at <= :end `;
      whereParams.start = ac.start;
      whereParams.end = ac.end;
    }
    sql += ` GROUP BY typeCode,uapp_id`;
    let result = await this.customQuery(sql,whereParams);
    return system.getResultSuccess(result);
  }

  /**
   * 需求统计（区域维度）
   * @param pobj
   * @returns {Promise<void>}
   */
  async getStatisticsByCity(pobj){
    let ac = pobj.actionBody;
    let sql = `select typeCode,province,count(*) count from n_need_info where province is not null and typeCode is not null  `;
    let whereParams = {};
    if(ac.start&&ac.end){
      sql += ` and created_at >= :start and created_at <= :end `;
      whereParams.start = ac.start;
      whereParams.end = ac.end;
    }
    if(ac.type_code){
      sql += ` and typeCode = :typeCode`;
      whereParams.typeCode = ac.type_code;
    }
    if(ac.uapp_id){
      sql += ` and uapp_id = :uapp_id`;
      whereParams.uapp_id = ac.uapp_id;
    }
    sql += ` GROUP BY province,typeCode`;
    let result = await this.customQuery(sql,whereParams);
    return system.getResultSuccess(result);
  }

  /**
   * 需求对比
   * @param pobj
   * @returns {Promise<void>}
   */
  async getNeedComparison(pobj){
    let ab =pobj.actionBody;
    let sql1 = `select count(*) count from n_need_info where deleted_at is null `;
    let sql2 = `select count(DISTINCT(a.needNo)) count from n_need_solution a left join n_need_info b on a.needNo = b.needNo where a.deleted_at is null and b.status = 'ycd'`;
    let whereParams = {};
    if(ab.startNow&&ab.endNow){
      sql1 += ` and created_at >= :start and created_at <= :end`;
      sql2 += ` and b.created_at >= :start and b.created_at <= :end`;
      whereParams.start = ab.startNow;
      whereParams.end = ab.endNow;
    }
    if(ab.uapp_id){
      sql1 += ` and uapp_id = :uapp_id`;
      sql2 +=  ` and b.uapp_id = :uapp_id`;
      whereParams.uapp_id = ab.uapp_id;
    }
    if(ab.type_code){
      sql1 += ` and typeCode = :type_code`;
      sql2 += ` and b.typeCode = :type_code`;
      whereParams.type_code = ab.type_code;
    }
    let totalRet1 = await this.customQuery(sql1,whereParams);
    let orderRet1 = await this.customQuery(sql2,whereParams);
    if(ab.startLast&&ab.endLast){
      whereParams.start = ab.startLast;
      whereParams.end = ab.endLast;
    }
    let totalRet2 = await this.customQuery(sql1,whereParams);
    let orderRet2 = await this.customQuery(sql2,whereParams);
    let result ={
      now:{
        total:totalRet1[0].count,
        order:orderRet1[0].count
      },
      last:{
        total:totalRet2[0].count,
        order:orderRet2[0].count
      }
    }
    return system.getResultSuccess(result);
  }
}
module.exports = NeedinfoService;

// var a=new NeedinfoService();
// var b=a.getItemByChannelSolutionNo({actionBody:{bizId:"1593141330846"}});